@@ -2,13 +2,14 @@ package Catalyst::Engine::PSGI;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Moose;
 extends 'Catalyst::Engine';
 
 use Scalar::Util qw(blessed);
 use URI;
+use Catalyst::Controller::Metal;
 
 # This is what Catalyst does to decode path. Not compatible to CGI RFC 3875
 my %reserved = map { sprintf('%02x', ord($_)) => 1 } split //, $URI::reserved;
@@ -127,6 +128,14 @@ sub read_chunk {
 sub run {
     my($self, $class, $env) = @_;
 
+    # short circuit with Metal
+    for my $metal (Catalyst::Controller::Metal->metals_for($class)) {
+        my $res = $metal->call($env);
+        if (defined $res && !(ref $res eq 'ARRAY' && $res->[0] == 404)) {
+            return $res;
+        }
+    }
+
     # what Catalyst->handle_request does
     my $status = -1;
     my $c;
@@ -136,6 +145,9 @@ sub run {
         $status = $c->finalize;
     };
 
+    # clear the $env ref to avoid leaks
+    $self->env(undef);
+
     if (my $error = $@) {
         chomp $error;
         $class->log->error(qq/Caught exception in engine "$error"/);
